using PropertyT

using SCS.SCSSolver
# using Mosek
# using CSDP
# using SDPA

scs_solver(tol, iterations) = SCSSolver(eps=tol, max_iters=iterations, linearsolver=SCS.Direct, alpha=1.95, acceleration_lookback=1)

# solver = Mosek.MosekSolver(
#    MSK_DPAR_INTPNT_CO_TOL_REL_GAP=tol,
#    MSK_IPAR_INTPNT_MAX_ITERATIONS=iterations,
#    QUIET=false)

# solver = CSDP.CSDPSolver(axtol=tol, atytol=tol, objtol=tol, minstepp=tol*10.0^-1, minstepd=tol*10.0^-1)

# solver = SDPA.SDPASolver(epsilonStar=tol, epsilonDash=tol)

include("FPGroups_GAP.jl")

include("groups/Allgroups.jl")
using PropertyTGroups

function summarize(groupdir, iterations, tol, upper_bound, radius, G, S)
    info("Group:       $groupdir")
    info("Iterations:  $iterations")
    info("Precision:   $tol")
    info("Upper bound: $upper_bound")
    info("Radius:      $radius")
    info("Threads:     $(Threads.nthreads())")
    info("Workers:     $(workers())")
    info(string(G))
    info("with generating set of size $(length(S))")
end

function params(Gr::PropertyTGroup)
    radius = Gr.args["radius"]
    tol = Gr.args["tol"]
    iterations = Gr.args["iterations"]
    upper_bound = Gr.args["upper-bound"]
    warm = Gr.args["warmstart"]
    return radius, tol, iterations, upper_bound, warm
end

function Settings(Gr::PropertyTGroup)
    r = Gr.args["radius"]
    ub = Gr.args["upper-bound"]
    groupdir = "$(PropertyTGroups.name(Gr))_r$r"

    radius, tol, iterations, upper_bound, warm = params(Gr)

    G = PropertyTGroups.group(Gr)
    S = PropertyTGroups.generatingset(Gr)

    summarize(groupdir, iterations, tol, upper_bound, radius, G, S)

    solver = scs_solver(tol, iterations)

    return PropertyT.Settings(groupdir, G, S, radius,
        solver, upper_bound, tol, warm)
end

function main(Gr::SymmetrizedGroup)
    sett = Settings(Gr)

    isdir(PropertyT.fullpath(sett)) || mkpath(PropertyT.fullpath(sett))

    if Gr.args["nosymmetry"]
        return PropertyT.check_property_T(PropertyT.Naive, sett)
    else
        autS = PropertyTGroups.autS(Gr)
        info("Symmetrising with $(autS)")
        sett.autS = autS
        return PropertyT.check_property_T(PropertyT.Symmetrize, sett)
    end
end

function main(Gr::GAPGroup)
    sett = Settings(Gr)

    S = [s for s in sett.S if s.symbols[1].pow == 1]
    relations = [k*inv(v) for (k,v) in sett.G.rels]

    prepare_pm_delta(PropertyT.prepath(sett), GAP_groupcode(S, relations), sett.radius)

    return PropertyT.check_property_T(PropertyT.Naive, sett)
end
